package com.gic.haoban.manage.service.service.hm.impl;

import com.gic.api.base.commons.Page;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.enums.hm.HmPageStatus;
import com.gic.haoban.manage.api.enums.hm.HmPageType;
import com.gic.haoban.manage.service.dao.mapper.hm.TabHaobanHmPageRelationMapper;
import com.gic.haoban.manage.service.entity.hm.TabHaobanHmPageRelation;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageRelationBO;
import com.gic.haoban.manage.service.pojo.qo.HmPageRelationQO;
import com.gic.haoban.manage.service.service.hm.HmPageRelationService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author MUSI
 * @Date 2022/7/3 9:45 PM
 * @Description
 * @Version
 **/
@Component
public class HmPageRelationServiceImpl implements HmPageRelationService {

    private static final Logger log = LoggerFactory.getLogger(HmPageRelationServiceImpl.class);

    @Autowired
    private TabHaobanHmPageRelationMapper haobanHmPageRelationMapper;


    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public boolean savePageRelation(HmPageRelationBO hmPageRelationBO) {
        TabHaobanHmPageRelation pageRelation = new TabHaobanHmPageRelation();
        pageRelation.setEnterpriseId(hmPageRelationBO.getEnterpriseId());
        pageRelation.setWxEnterpriseId(hmPageRelationBO.getWxEnterpriseId());
        pageRelation.setPageCode(hmPageRelationBO.getPageCode());
        pageRelation.setReferId(hmPageRelationBO.getReferId());
        pageRelation.setReferName(hmPageRelationBO.getReferName());
        pageRelation.setCreateTime(new Date());
        pageRelation.setUpdateTime(new Date());
        pageRelation.setType(HmPageType.LINK.getCode());
        pageRelation.setStatus(HmPageStatus.ENABLE.getCode());
        return haobanHmPageRelationMapper.insertSelective(pageRelation) == 1;
    }

    @Override
    public boolean updateRelationStatus(HmPageRelationBO hmPageRelationBO) {
        // 移除该referId之前的引用
        Date now = new Date();
        HmPageRelationQO pageRelationQo = new HmPageRelationQO();
        pageRelationQo.setStatus(HmPageStatus.ENABLE.getCode());
        pageRelationQo.setReferId(hmPageRelationBO.getReferId());
        List<TabHaobanHmPageRelation> tabHaobanHmPageRelations = haobanHmPageRelationMapper.queryPageRelationList(pageRelationQo);
        if (CollectionUtils.isNotEmpty(tabHaobanHmPageRelations)){
            for (TabHaobanHmPageRelation tabHaobanHmPageRelation : tabHaobanHmPageRelations) {
                tabHaobanHmPageRelation.setStatus(HmPageStatus.DISABLE.getCode());
                tabHaobanHmPageRelation.setUpdateTime(now);
                haobanHmPageRelationMapper.updateByPrimaryKey(tabHaobanHmPageRelation);
            }
        }
        // insert new relation
        return this.savePageRelation(hmPageRelationBO);
    }

    @Override
    public List<HmPageRelationBO> queryPageRelation(String pageCode) {
        HmPageRelationQO hmPageRelationQo = new HmPageRelationQO();
        hmPageRelationQo.setPageCode(pageCode);
        hmPageRelationQo.setStatus(HmPageStatus.ENABLE.getCode());
        hmPageRelationQo.setPageSize(1);
        PageHelper.startPage(hmPageRelationQo.getPageNum(), hmPageRelationQo.getPageSize());
        List<TabHaobanHmPageRelation> tabHaobanHmPageRelations =
                haobanHmPageRelationMapper.queryPageRelationList(hmPageRelationQo);
        return tabHaobanHmPageRelations.stream().map(item -> {
            HmPageRelationBO temp = new HmPageRelationBO();
            BeanUtils.copyProperties(item, temp);
            return temp;
        }).collect(Collectors.toList());
    }

    /**
     * 链接code/模板名称 条件查询
     *
     * @return
     */
    @Override
    public Page<HmPageRelationBO> queryPageRelationWithSearchText(HmPageRelationQO hmPageRelationQo) {
        PageHelper.startPage(hmPageRelationQo.getPageNum(), hmPageRelationQo.getPageSize());
        com.github.pagehelper.Page<TabHaobanHmPageRelation> pageRelationPage =
                (com.github.pagehelper.Page<TabHaobanHmPageRelation>) haobanHmPageRelationMapper.queryPageRelationList(hmPageRelationQo);
        return PageUtil.changePageHelperToCurrentPage(pageRelationPage, HmPageRelationBO.class);
    }

    /**
     * 统计templateCode 被引用的总数
     *
     * @param pageCode
     * @return
     */
    @Override
    public Integer countPageTemplateRelation(String pageCode) {
        return null;
    }
}
