package com.gic.haoban.manage.service.service.impl;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.dto.KeyDataDTO;
import com.gic.haoban.manage.api.enums.KeyDataEnum;
import com.gic.haoban.manage.service.dao.mapper.TabKeyDataMapper;
import com.gic.haoban.manage.service.entity.TabKeyData;
import com.gic.haoban.manage.service.service.KeyDataService;

@Service("keyDataService")
public class KeyDataServiceImpl implements KeyDataService {

    @Autowired
    private TabKeyDataMapper tabKeyDataMapper;

    @Override
    public KeyDataDTO saveData(String enterpriseId, String relationId, String data, KeyDataEnum dataEnum) {
        KeyDataDTO dto = new KeyDataDTO();
        dto.setRelationId(relationId);
        dto.setEnterpriseId(enterpriseId);
        dto.setDataKey(generateKey(dataEnum, data));
        dto.setCreateTime(new Date());
        dto.setId(UniqueIdUtils.uniqueLong());
        dto.setData(data);
        dto.setType(dataEnum.getType());
        TabKeyData entity = EntityUtil.changeEntityByJSON(TabKeyData.class, dto);
        this.tabKeyDataMapper.insert(entity);
        return dto;
    }

    @Override
    public String saveHmData(String enterpriseId, String unionid, Long linkId, Long hmId) {
        JSONObject json = new JSONObject();
        json.put("unid", unionid);
        json.put("ljid", linkId);
        json.put("hmid", hmId);
        KeyDataDTO data = this.saveData(enterpriseId, String.valueOf(linkId), json.toString(), KeyDataEnum.DYNAMIC_HM);
        return data.getDataKey();
    }

    @Override
    public KeyDataDTO getDataByKey(String key, KeyDataEnum dataEnum, String relationId) {
        TabKeyData tab = this.tabKeyDataMapper.selectByKey(key, dataEnum.getType(), relationId);
        if (null != tab) {
            return EntityUtil.changeEntityByJSON(KeyDataDTO.class, tab);
        }
        return null;
    }

    @Override
    public String getMobile(String key, String relationId) {
        KeyDataDTO data = this.getDataByKey(key, KeyDataEnum.MOBILE, relationId);
        if (null != data) {
            return JSON.parseObject(data.getData()).getString("ph");
        }
        return null;
    }

    public static String generateKey(KeyDataEnum dataEnum, String data) {
        if (dataEnum.equals(KeyDataEnum.MOBILE)) {
            String phone = JSON.parseObject(data).getString("ph");
            return Long.toHexString(Long.valueOf(phone));
        }
        return UniqueIdUtils.uniqueLongHex();
    }

}
