package com.gic.haoban.manage.service.service.out.impl.hm;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.hm.HmGroupDTO;
import com.gic.haoban.manage.api.dto.hm.HmGroupQueryDTO;
import com.gic.haoban.manage.api.enums.hm.HmGroupStatus;
import com.gic.haoban.manage.api.service.hm.HmGroupApiService;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmGroupSettingBO;
import com.gic.haoban.manage.service.pojo.qo.HmGroupInfoQO;
import com.gic.haoban.manage.service.service.hm.HmGroupService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author MUSI
 * @Date 2022/7/1 4:02 PM
 * @Description
 * @Version
 **/
@Component
public class HmGroupApiServiceImpl implements HmGroupApiService {

    private static final Logger log = LoggerFactory.getLogger(HmGroupApiServiceImpl.class);

    private static final Long DEFAULT_GROUP_ID = NumberUtils.LONG_ZERO;
    private static final String DEFAULT_GROUP_NAME = "默认分组";

    @Autowired
    private HmGroupService groupService;

    @Override
    public ServiceResponse<Long> saveOrUpdateHmGroupSetting(HmGroupDTO hmGroupDTO) {
        log.info("[saveOrUpdateHmGroupSetting] params:{}", JSON.toJSONString(hmGroupDTO));
        if (StringUtils.isBlank(hmGroupDTO.getGroupName())) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_5.getCode() + "", HaoBanErrCodeCommon.ERR_5.getMsg());
        }
        if (StringUtils.equals(DEFAULT_GROUP_NAME, hmGroupDTO.getGroupName())) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
        }
        HmGroupInfoQO groupInfoQo = new HmGroupInfoQO();
        groupInfoQo.setGroupName(hmGroupDTO.getGroupName());
        groupInfoQo.setStatus(HmGroupStatus.ENABLE.getCode());
        groupInfoQo.setEnterpriseId(hmGroupDTO.getEnterpriseId());
        groupInfoQo.setWxEnterpriseId(groupInfoQo.getWxEnterpriseId());
        Page<HmGroupSettingBO> hmGroupSettingPage = groupService.queryGroupSettingList(groupInfoQo);
        if (hmGroupDTO.getGroupId() == null) {
            // insert
            // check groupName unique
            if (CollectionUtils.isNotEmpty(hmGroupSettingPage.getResult())) {
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_5.getCode() + "", HaoBanErrCodeCommon.ERR_5.getMsg());
            }
        } else {
            // update
            if (CollectionUtils.isNotEmpty(hmGroupSettingPage.getResult())
                    && hmGroupSettingPage.getTotalCount() > 1) {
                // 名称重复
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_5.getCode() + "", HaoBanErrCodeCommon.ERR_5.getMsg());
            }
            if (CollectionUtils.isNotEmpty(hmGroupSettingPage.getResult())
                    && !hmGroupSettingPage.getResult().get(0).getGroupId().equals(hmGroupDTO.getGroupId())) {
                // 名称重复
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_5.getCode() + "", HaoBanErrCodeCommon.ERR_5.getMsg());
            }
            HmGroupSettingBO groupSettingBo = groupService.queryGroupSettingDetail(hmGroupDTO.getGroupId());
            if (groupSettingBo == null) {
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
            }
            if (!StringUtils.equals(groupSettingBo.getEnterpriseId(), hmGroupDTO.getEnterpriseId())) {
                // 无权限
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
            }
        }
        Long groupId = groupService.saveOrUpdateGroupSetting(hmGroupDTO);
        return ServiceResponse.success(groupId);
    }

    @Override
    public ServiceResponse<Page<HmGroupDTO>> queryHmGroupSettingList(HmGroupQueryDTO groupQueryDTO) {

        HmGroupInfoQO groupInfoQo = new HmGroupInfoQO();
        BeanUtils.copyProperties(groupQueryDTO, groupInfoQo);
        groupInfoQo.setStatus(HmGroupStatus.ENABLE.getCode());
        Page<HmGroupSettingBO> groupSettingPage = groupService.queryGroupSettingList(groupInfoQo);

        Page<HmGroupDTO> groupDtoPage = PageUtil.changeToCurrentPage(groupSettingPage, HmGroupDTO.class);
        List<HmGroupDTO> result = new ArrayList<>();
        if (groupQueryDTO.getPageNum() == 1) {
            // 第一页展示默认分组选项
            // todo query 活码 默认分组的总数
            result.add(buildDefault(0));
        }
        if (CollectionUtils.isNotEmpty(groupDtoPage.getResult())) {
            groupDtoPage.getResult().forEach(item -> item.setDefaultFlag(0));
            result.addAll(groupDtoPage.getResult());
        }
        groupDtoPage.setResult(result);
        return ServiceResponse.success(groupDtoPage);
    }

    private HmGroupDTO buildDefault(Integer num) {
        HmGroupDTO groupSettingDTO = new HmGroupDTO();
        groupSettingDTO.setGroupId(DEFAULT_GROUP_ID);
        groupSettingDTO.setGroupName(DEFAULT_GROUP_NAME);
        groupSettingDTO.setDefaultFlag(1);
        groupSettingDTO.setReferNum(num);
        return groupSettingDTO;
    }

    @Override
    public ServiceResponse<Boolean> deleteGroupSetting(HmGroupDTO groupDTO) {
        log.info("[deleteGroupSetting] 删除分组, params:{}", JSON.toJSONString(groupDTO));
        if (groupDTO.getGroupId() == null) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_2.getCode() + "", HaoBanErrCodeCommon.ERR_2.getMsg());
        }
        if (DEFAULT_GROUP_ID.equals(groupDTO.getGroupId())) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_5.getCode() + "", HaoBanErrCodeCommon.ERR_5.getMsg());
        }
        HmGroupSettingBO groupSettingBo = groupService.queryGroupSettingDetail(groupDTO.getGroupId());
        if (groupSettingBo == null
                || !HmGroupStatus.ENABLE.getCode().equals(groupSettingBo.getStatus())){
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
        }
        if (groupSettingBo.getReferNum() != null && groupSettingBo.getReferNum() > 0){
            return ServiceResponse.failure(HaoBanErrCode.ERR_100033.getCode(), HaoBanErrCode.ERR_100033.getMsg());
        }
        groupService.deleteGroupSetting(groupDTO);
        return ServiceResponse.success(Boolean.TRUE);
    }

    /**
     * 分组总引用次数
     *
     * @param enterpriseId
     * @param wxEnterpriseId
     * @return
     */
    @Override
    public ServiceResponse<Integer> queryGroupTotalReferCount(String enterpriseId, String wxEnterpriseId) {
        Integer referCount = groupService.queryGroupTotalReferCount(enterpriseId, wxEnterpriseId);
        return ServiceResponse.success(referCount);
    }

    /**
     * 查询单个分组信息
     *
     * @param groupId
     * @return
     */
    @Override
    public ServiceResponse<HmGroupDTO> queryGroupDetail(Long groupId) {
        HmGroupSettingBO groupSettingBO = groupService.queryGroupSettingDetail(groupId);
        HmGroupDTO groupDTO = EntityUtil.changeEntityByOrika(HmGroupDTO.class, groupSettingBO);
        return ServiceResponse.success(groupDTO);
    }
}
