package com.gic.haoban.manage.service.service.out.impl.hm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreSettingDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkWxaDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.api.service.hm.HmLinkApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmLinkBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.service.KeyDataService;
import com.gic.haoban.manage.service.service.MemberQueryService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmLinkStoreService;
import com.gic.haoban.manage.service.service.hm.HmPageService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.member.api.dto.es.MemberDataDTO;
import com.gic.member.api.dto.es.MemberStoreClerkDataDTO;
import com.gic.member.api.service.MemberService;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactResultDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

@Service("hmLinkApiService")
public class HmLinkApiServiceImpl implements HmLinkApiService {

    private static final Logger log = LoggerFactory.getLogger(HmLinkApiService.class);

    @Autowired
    private HmLinkService hmLinkService;
    @Autowired
    private HmLinkStoreService hmLinkStoreService;
    @Autowired
    private KeyDataService keyDataService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private MemberQueryService memberQueryService;
    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private Config config;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private HmPageService hmPageService;

    @Override
    public ServiceResponse<HmLinkDTO> getByLinkId(String wxEnterpriseId, String enterpriseId, Long linkId) {
        HmLinkBO link = this.hmLinkService.getByLinkId(wxEnterpriseId, enterpriseId, linkId);
        if (null != link) {
            HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, link);
            List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(dto.getWxEnterpriseId(),
                    dto.getEnterpriseId(), linkId);
            dto.setStoreList(storeList);
            return ServiceResponse.success(dto);
        }
        return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
    }

    @Override
    public ServiceResponse<HmLinkDTO> getByShortCode(String shortCode) {
        HmLinkBO bo = this.hmLinkService.getByLinkShortCode(shortCode);
        if (null == bo) {
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
        }
        HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, bo);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<List<HmLinkDTO>> listForSms(String wxEnterpriseId, String enterprieId, String linkName,
            Integer linkType) {
        List<HmLinkBO> list = this.hmLinkService.listForSms(wxEnterpriseId, enterprieId, linkName, linkType);
        List<HmLinkDTO> retList = EntityUtil.changeEntityListByJSON(HmLinkDTO.class, list);
        return ServiceResponse.success(retList);
    }

    @Override
    public ServiceResponse<String> save(HmLinkDTO dto) {
        String id = this.hmLinkService.saveLink(dto, dto.getStoreList());
        return ServiceResponse.success(id);
    }

    @Override
    public ServiceResponse<Page<HmLinkDTO>> listPage(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        Page<HmLinkBO> page = this.hmLinkService.listPage(qdto, basePageInfo);
        Page<HmLinkDTO> resultPage = PageHelperUtils.changePageToCurrentPage(page, HmLinkDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<HmLinkWxaDTO> getLinkHmFromWxa(String enterpriseId, Integer whitchStore, String inStoreId,
            String inClerkId, String linkShortCode, String mixPhone, String unionid) {
        HmQrcodeBO hm = null;
        HmLinkWxaDTO retDTO = new HmLinkWxaDTO();
        log.info("小程序获取活码={},linkShortCode={},mixPhone={}", enterpriseId, linkShortCode, mixPhone);
        HmLinkBO link = this.hmLinkService.getByLinkShortCode(linkShortCode);
        if (null == link) {
            log.error("链接记录不存,linkShortCode={}", linkShortCode);
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接记录不存");
        }
        Long linkId = link.getLinkId();
        String wxEnterpriseId = link.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            log.error("未查询到微信企业,id={}", wxEnterpriseId);
            return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "未查询到微信企业");
        }
        // 判断导购是否有活码,如果没有返回错误
        if (StringUtils.isNotBlank(inClerkId)) {
            hm = this.getHmFromClerk(inClerkId, wxEnterpriseId);
            if (null == hm) {
                return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
            }
        }
        // 直接返回门店的导购活码,如果没有返回错误
        if (null == hm && StringUtils.isNotBlank(inStoreId)) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                int dz = 0, dy = 0;
                // 1推荐门店 2附件门店 3省市区门店
                int storeType = storeRuleObj.getIntValue("store_type");
                // 省市区
                if (whitchStore == 3) {
                    HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                    dz = cityStore.getDz();
                    dy = cityStore.getDy();
                }
                if (whitchStore == 1 || whitchStore == 2) {
                    HmLinkStoreSettingDTO store = storeRuleObj.getObject("store", HmLinkStoreSettingDTO.class);
                    if (whitchStore == 2 || (storeType == 1 && whitchStore == 1)) {
                        // 附近门店 或者 推荐门店
                        dz = store.getDz();
                        dy = store.getDy();
                    } else if (storeType == 2 && whitchStore == 1) {
                        // 附近门店的推荐门店
                        dz = store.getNdz();
                        dy = store.getNdy();
                    }
                }
                hm = this.getHmFormStore(wxEnterpriseId, inStoreId, dz, dy);
                log.info("获取到的门店配置dz={},dy={},获取的活码={}", dz, dy, JSON.toJSONString(hm));
                if (null == hm) {
                    return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
                }
            }
        }
        // 会员门店
        if (null == hm && StringUtils.isNotEmpty(mixPhone)) {
            HmLinkStoreSettingDTO fwStore = null;
            HmLinkStoreSettingDTO xgStore = null;
            if (StringUtils.isNotBlank(link.getStoreRuleJson())) {
                String storeRuleJson = link.getStoreRuleJson();
                log.info("会员门店规则={}", storeRuleJson);
                JSONObject storeRuleObj = JSONObject.parseObject(storeRuleJson);
                fwStore = storeRuleObj.getObject("fw_store", HmLinkStoreSettingDTO.class);
                if (null == fwStore || fwStore.getOpen() == 0) {
                    fwStore = null;
                }
                xgStore = storeRuleObj.getObject("xg_store", HmLinkStoreSettingDTO.class);
                if (null == xgStore || xgStore.getOpen() == 0) {
                    xgStore = null;
                }
                String mobile = this.keyDataService.getMobile(mixPhone, String.valueOf(linkId));
                log.info("小程序获取活码mobile={}", mobile);
                String memberId = this.memberService.getMemberIdByPhoneNumber(mobile, enterpriseId);
                log.info("小程序获取活码memberId={}", memberId);
                MemberDataDTO member = this.getMemberClerk(enterpriseId, memberId);
                String mainClerkId = member.getMainClerkId();
                String mainStoreId = member.getMainStoreId();
                // 服务门店
                if (null != fwStore) {
                    if (fwStore.getZs() == 1) {
                        hm = this.getHmFromClerk(mainClerkId, wxEnterpriseId);
                    }
                    if (null == hm) {
                        hm = this.getHmFormStore(wxEnterpriseId, mainStoreId, fwStore.getDz(), fwStore.getDy());
                    }
                }
                // 协管门店
                if (null == hm && null != xgStore) {
                    List<MemberStoreClerkDataDTO> list = member.getStoreClerkRel();
                    List<String> storeIdList = new ArrayList<>();
                    // 协管导购
                    for (MemberStoreClerkDataDTO item : list) {
                        String clerkId = item.getClerkIdRel();
                        if (clerkId.equals(mainClerkId)) {
                            continue;
                        }
                        hm = this.getHmFromClerk(clerkId, wxEnterpriseId);
                        if (null != hm) {
                            break;
                        }
                        storeIdList.add(item.getStoreIdRel());
                    }
                    // 店长
                    if (null == hm && xgStore.getDz() == 1) {
                        for (String storeId : storeIdList) {
                            hm = this.getHmFormStore(wxEnterpriseId, storeId, 1, 0);
                            if (null != hm) {
                                break;
                            }
                        }
                    }
                    // 店员
                    if (null == hm && xgStore.getDy() == 1) {
                        for (String storeId : storeIdList) {
                            hm = this.getHmFormStore(wxEnterpriseId, storeId, 0, 1);
                            if (null != hm) {
                                break;
                            }
                        }
                    }
                }
            }
        }
        if (null == hm && StringUtils.isNotBlank(link.getCustomRuleJson())) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                // 门店规则1推荐门店 2附件门店
                int storeType = storeRuleObj.getIntValue("store_type");
                List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(wxEnterpriseId, enterpriseId,
                        linkId);
                log.info("门店规则={},推荐门店列表={}", storeType, JSON.toJSONString(storeList));
                HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                // 是否开启省市区门店查询
                int selectStoreFlag = cityStore.getOpen();
                log.info("省市区开启={}", selectStoreFlag);
                retDTO.setStoreType(storeType);
                retDTO.setStoreList(storeList);
                retDTO.setSelectStoreFlag(selectStoreFlag);
            }
        }
        if (null != hm) {
            String key = this.keyDataService.saveHmData(enterpriseId, unionid, linkId, hm.getHmId());
            if (null != key) {
                this.addQrcode(hm, key, wxEnterpriseDTO.getCorpid(), retDTO);
            }
            // 查询页面
            HmPageBO page = this.hmPageService.queryHmPageDetail(link.getPageId());
            // retDTO.setPage(resp.getResult());
        }
        // 生成动态活码
        log.info(JSON.toJSONString(retDTO, true));
        return ServiceResponse.success(retDTO);
    }

    private void addQrcode(HmQrcodeBO hm, String key, String corpid, HmLinkWxaDTO retDTO) {
        QywxExternalcontactDTO paramsDTO = new QywxExternalcontactDTO();
        // 联系方式类型,1-单人, 2-多人
        paramsDTO.setType(1);
        // 场景，1-在小程序中联系，2-通过二维码联系
        paramsDTO.setScene(2);
        paramsDTO.setRemark(hm.getName());
        paramsDTO.setSkipVerify(hm.getPassFlag() == 1);
        paramsDTO.setState("DT" + key);
        paramsDTO.setTemp(true);
        // 过期时间一小时
        paramsDTO.setExpiresIn(60L * 60);
        paramsDTO.setUser(Arrays.asList(hm.getWxUserId()));
        QywxExternalcontactResultDTO dto = qywxUserApiService.addContactWay(corpid, config.getWxSuiteid(), paramsDTO);
        log.info("创建动态活码参数={},返回={}", corpid, JSON.toJSONString(paramsDTO), JSON.toJSONString(dto));
        if (null != dto) {
            retDTO.setHmQrcode(retDTO.getHmQrcode());
        }
    }

    private HmQrcodeBO getHmFromClerk(String clerkId, String wxEnterpriseId) {
        if (StringUtils.isBlank(clerkId) || "-1".equals(clerkId)) {
            return null;
        }
        HmQrcodeBO hm = this.hmQrcodeService.queryByClerkId(clerkId, wxEnterpriseId, 0);
        return hm;
    }

    private HmQrcodeBO getHmFormStore(String wxEnterpriseId, String inStoreId, int dz, int dy) {
        if (StringUtils.isBlank(inStoreId) || "-1".equals(inStoreId)) {
            return null;
        }
        HmQrcodeBO hm = null;
        if (dz == 1) {
            ClerkDTO dzClerk = this.clerkService.getClerkLeaderByStoreId(inStoreId);
            if (null != dzClerk) {
                hm = this.getHmFromClerk(dzClerk.getClerkId(), wxEnterpriseId);
            } else {
                log.info("店长不存在,storeId={}", inStoreId);
            }
        }
        if (null == hm && dy == 1) {
            List<HmQrcodeBO> list = this.hmQrcodeService.listByStoreId(wxEnterpriseId, inStoreId, 0);
            if (CollectionUtils.isNotEmpty(list)) {
                hm = list.get(new Random().nextInt(list.size()));
            }
        }
        return hm;
    }

    // 冗余会员信息
    private MemberDataDTO getMemberClerk(String enterpriseId, String memberId) {
        String returnFields = "memberId,mainClerkId,mainStoreId,storeClerkRel";
        JSONObject memberObj = this.memberQueryService.queryDataSingle(enterpriseId, memberId, returnFields);
        if (null == memberObj) {
            return null;
        }
        log.info("会员信息={}", memberObj.toJSONString());
        MemberDataDTO memberDataDTO = memberObj.toJavaObject(MemberDataDTO.class);
        return memberDataDTO;
    }
}
