package com.gic.haoban.manage.service.service.out.impl.hm;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.hm.HmPageDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmPageRelationQDTO;
import com.gic.haoban.manage.api.enums.hm.HmPageStatus;
import com.gic.haoban.manage.api.service.hm.HmPageApiService;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageRelationBO;
import com.gic.haoban.manage.service.pojo.qo.HmPageRelationQO;
import com.gic.haoban.manage.service.pojo.qo.HmPageTemplateQO;
import com.gic.haoban.manage.service.service.hm.HmPageRelationService;
import com.gic.haoban.manage.service.service.hm.HmPageService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * @Author MUSI
 * @Date 2022/7/3 8:22 PM
 * @Description
 * @Version
 **/
@Component(value = "hmPageTemplateApiServiceImpl")
public class HmPageApiServiceImpl implements HmPageApiService {

    private static final Logger log = LoggerFactory.getLogger(HmPageApiServiceImpl.class);
    @Autowired
    private HmPageRelationService pageRelationService;

    @Autowired
    private HmPageService haobanHmPageService;

    @Override
    public ServiceResponse<Long> saveOrUpdateHmPage(HmPageDTO pageDTO) {

        if (Objects.nonNull(pageDTO.getPageId())) {
            // update
            HmPageBO pageDetail = haobanHmPageService.queryHmPageDetail(pageDTO.getPageId());
            if (Objects.isNull(pageDetail)) {
                // 数据不存在
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
            }
            if (!StringUtils.equals(pageDetail.getEnterpriseId(), pageDTO.getEnterpriseId())) {
                // 无权限
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
            }
            if (!StringUtils.equals(pageDetail.getWxEnterpriseId(), pageDTO.getWxEnterpriseId())) {
                // 无权限
                return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
            }
        }
        return ServiceResponse.success(haobanHmPageService.saveOrUpdateHmPage(pageDTO));
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> deleteHmPage(HmPageDTO pageDTO) {
        log.info("[deleteHmPage] 删除落地页模板, params:{}", JSON.toJSONString(pageDTO));
        HmPageBO pageBo = haobanHmPageService.queryHmPageDetail(pageDTO.getPageId());
        if (Objects.isNull(pageBo) || !Objects.equals(HmPageStatus.ENABLE.getCode(), pageBo.getStatus())) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_8.getCode() + "", HaoBanErrCodeCommon.ERR_8.getMsg());
        }
        if (this.checkPageIsBeQuoted(pageBo.getPageCode()) > 0) {
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_9.getCode() + "", HaoBanErrCodeCommon.ERR_9.getMsg());
        }
        haobanHmPageService.deleteHmPage(pageDTO);
        return ServiceResponse.success(Boolean.TRUE);
    }

    @Override
    public ServiceResponse<Page<HmPageDTO>> queryPageList(HmPageQDTO pageQueryDTO) {
        HmPageTemplateQO pageTemplateQo = new HmPageTemplateQO();
        BeanUtils.copyProperties(pageQueryDTO, pageTemplateQo);
        pageTemplateQo.setStatus(HmPageStatus.ENABLE.getCode());
        Page<HmPageBO> hmPageBoPages = haobanHmPageService.queryPageList(pageTemplateQo);
        Page<HmPageDTO> pageRespVos = PageUtil.changeToCurrentPage(hmPageBoPages, HmPageDTO.class);
        return ServiceResponse.success(pageRespVos);
    }

    /**
     * 校验落地页模板pageTemplateCode 是否被关联
     *
     * @param pageCode
     * @return true 存在关联
     * false 不存在关联关系
     */
    private Integer checkPageIsBeQuoted(String pageCode) {
        List<HmPageRelationBO> relations = pageRelationService.queryPageRelation(pageCode);
        return 0;
    }

    /**
     * 条件查询落地页关联详情
     *
     * @param relationQueryDTO
     * @return
     */
    @Override
    public ServiceResponse<Page<HmPageRelationDTO>> queryPageRelation(HmPageRelationQDTO relationQueryDTO) {
        HmPageRelationQO hmPageRelationQo = new HmPageRelationQO();
        hmPageRelationQo.setTemplateSearchText(relationQueryDTO.getSearchText());
        hmPageRelationQo.setEnterpriseId(relationQueryDTO.getEnterpriseId());
        hmPageRelationQo.setWxEnterpriseId(relationQueryDTO.getWxEnterpriseId());
        hmPageRelationQo.setPageNum(relationQueryDTO.getPageNum());
        hmPageRelationQo.setPageSize(relationQueryDTO.getPageSize());
        hmPageRelationQo.setPageCode(relationQueryDTO.getPageId() + "");
        hmPageRelationQo.setStatus(HmPageStatus.ENABLE.getCode());
        Page<HmPageRelationBO> hmPageRelationPage = pageRelationService.queryPageRelationWithSearchText(hmPageRelationQo);
        return ServiceResponse.success(PageUtil.changeToCurrentPage(hmPageRelationPage, HmPageRelationDTO.class));
    }

    /**
     * 根据页面id 查询
     *
     * @param pageId
     * @return
     */
    @Override
    public ServiceResponse<HmPageDTO> queryPageDetail(Long pageId) {
        HmPageBO pageBO = haobanHmPageService.queryHmPageDetail(pageId);
        if (pageBO == null) {
            return ServiceResponse.success();
        }
        HmPageDTO hmPageDTO = new HmPageDTO();
        BeanUtils.copyProperties(pageBO, hmPageDTO);
        return ServiceResponse.success(hmPageDTO);
    }
}
